/* スクリプトコールの戻り値
*/ 
var FNC_RESULT = {
	OK : 0,				// 正常終了
	FAILED : 1,			// 一般的なエラー
	NOT_MATCH : 2,		// パラメータなどが一致しない
	MISS_VALUE : 3		// 値が無い
};

var UNSET_INDEX = -1;

//-outパラメータのデフォルト値
var DEFAULT_OUTPUT_DIR = "$(DetectedDataDir)\\$(MonitorID).txt";

/* BOMのCParamArrayクラス相当
*/ 
function CParamArrayJ() {
	// パラメータ格納クラス
	function CParam(srcKey, srcVal) {
		this.key = srcKey;
		this.val = srcVal;
	}

	// パラメータ格納配列
	this.m_Array = [];
	
	// パラメータ位置取得
	this.getParamIndex = function(strKey, aryIndex) {
		var result = -1;
		var minCtr = 0;
		if (aryIndex >= 0) minCtr = aryIndex;
		
		for (var ctr = minCtr ; ctr < this.m_Array.length ; ++ctr) {
			if (this.m_Array[ctr].key == strKey) {
				result = ctr;
				break;
			}
		}
		
		return result;
	};
	
	// パラメータ取得
	this.getValue = function(strKey, aryIndex) {
		var result = "";
		var index  = this.getParamIndex(strKey, aryIndex);
		
		if (index >= 0) {
			result = this.m_Array[index].val;
		}
		
		return result;
	};

	// パラメータ確認
	this.hasParam = function(strKey, aryIndex) {
		var index  = this.getParamIndex(strKey, aryIndex);
		if (index >= 0) {
			return true;
		} else {
			return false;
		}
	};

	// パラメータ設定
	this.setParam = function(strKey, strValue, aryIndex) {
		var index = this.getParamIndex(strKey, aryIndex);
		
		if (index >= 0) {
			this.m_Array[index].val = strValue;
		} else {
			this.m_Array.push(new CParam(strKey, strValue));
		}
	};

	// パラメータ追加
	this.addParam = function(strKey, strValue) {
		var prm = new CParam(strKey, strValue);
		this.m_Array.push(prm);
	};

	// 全キー取得
	this.getAllKeys = function() {
		var result = [];
		
		for (var ctr = 0 ; ctr < this.m_Array.length ; ++ctr) {
			result.push(this.m_Array[ctr].key);
		}
		
		return result;
	};
	
	// パラメータ初期化
	this.clearParam = function() {
		this.m_Array = [];
	};
	
	// パラメータ削除
	this.removeParam = function(strKey, aryIndex) {
		var rmvIdx = this.getParamIndex(strKey, aryIndex);

		if (aryIndex < 0) {
			// インデックス未指定は全削除
			while (rmvIdx >= 0) {
				this.m_Array.splice(rmvIdx, 1);
				rmvIdx = this.getParamIndex(strKey);
			}
		} else {
			// 指定があれば、１つだけ削除
			this.m_Array.splice(rmvIdx, 1);
			rmvIdx = this.getParamIndex(strKey);
		}
	};
	
	// 引数分割
	this.splitArg = function(strArg) {
		var sTmp = strArg;
		var result = null;
		if (sTmp.length < 1) return result;
		
		var bRpl = false;
		if (sTmp.match(/^\"/)) {
			sTmp = sTmp.replace(/^\"/, "");
			bRpl = true;
		}
		if (sTmp.match(/\"$/) && bRpl) {
			sTmp = sTmp.replace(/\"$/, "");
		}
		if (sTmp.match(/^\-/)) {
			sTmp = sTmp.replace(/^\-/, "");
		}
		
		var pt = sTmp.indexOf(":", 0);
		if (pt < 0) {
			// 区切りなし
			result = new CParam(sTmp, "");
		} else if (pt == 0) {
			// 先頭が区切り
			result = new CParam(sTmp, "");
		} else {
			result = new CParam(sTmp.slice(0, pt), sTmp.slice(pt + 1));
		}
		
		return result;
	};

	// コマンドライン解析
	this.parseCommand = function(strLine) {
		this.clearParam();
		if (strLine.length < 1) return;
		
		var bInToken = false;
		var bToken   = false;
		var strToken = "";
		
		for (var ctr = 0 ; ctr <= strLine.length ; ++ctr) {
			// 終端チェック
			if (ctr >= strLine.length) {
				if (bToken) {
					var p = this.splitArg(strToken);
					if (p) this.m_Array.push(p);
					strToken = "";
					bToken   = false;
					break;
				}
			}
			
			var ch = strLine.slice(ctr, ctr + 1);
			if (ch == "\\") {
				var strLast = strLine.slice(ctr);
				var nPt     = strLast.indexOf("\\", 0);
				if (strLast.slice(nPt + 1, nPt + 2) == "\"") {
					++ctr;
				}
				strToken += strLine.slice(ctr, ctr + 1);
				bToken   = true;
			} else if (ch == "\"") {
				bInToken = !bInToken;
				bToken   = !bInToken;
			} else if (!bInToken && ch == " ") {
				if (bToken) {
					var p = this.splitArg(strToken);
					if (p) this.m_Array.push(p);
					strToken = "";
					bToken   = false;
				}
			} else {
				strToken += ch;
				bToken   = true;
			}
		}
	};
	
	// コマンドライン作成
	this.makeCommand = function() {
		var result = "";
		var keys   = this.getAllKeys();

		// 処理ループ
		for (var ctr = 0 ; ctr < keys.length ; ++ctr) {
			if (result.length > 0) {
				result += " ";
			}
			
			// 値を取得
			var v = this.getValue(keys[ctr]);
			var para = "-" + keys[ctr];
			if (v.length > 0) {
				para += ":" + v;
			}
			para = "\"" + para + "\"";
			
			// パラメータを連結
			result += para;
		}
		
		return result;
	};
}



/* 本体とのI/Fタグ操作用クラス
*/ 
function CBomMonIF() {
	this.m_ErrMsg = "";

	// コマンド名取得
	this.getCommandName = function() {
		var result = "";
		
		var elm = document.getElementById('bcm-b-obj');
		if (elm) {
			result = elm.value;
		} else {
			this.m_ErrMsg = "対象のタグが見つかりませんでした。";
		}
		
		if (result == void 0) {
			result = "";
		}
		
		return result;
	};

	// パラメータライン取得
	this.getParameterLine = function () {
		var result = "";
		
		var elm = document.getElementById('bcm-b-prm');
		if (elm) {
			result = elm.value;
		} else {
			this.m_ErrMsg = "対象のタグが見つかりませんでした。";
		}
		
		if (result == void 0) {
			result = "";
		}
		
		return result;
	};

	// コマンド名設定
	this.setCommandName = function(cmdName) {
		var result = false;
		
		var elm = document.getElementById('bcm-b-obj');
		if (elm) {
			try {
				elm.value = cmdName;
				result = true;
			} catch (e) {
				this.m_ErrMsg = "値の設定でエラーが発生しました。(" + e + ")";
			}
		} else {
			this.m_ErrMsg = "対象のタグが見つかりませんでした。";
		}
		
		return result;
	};

	// パラメータライン設定
	this.setParameterLine = function(paraLine) {
		var result = false;
		
		var elm = document.getElementById('bcm-b-prm');
		if (elm) {
			try {
				elm.value = paraLine;
				result = true;
			} catch (e) {
				this.m_ErrMsg = "値の設定でエラーが発生しました。(" + e + ")";
			}
		} else {
			this.m_ErrMsg = "対象のタグが見つかりませんでした。";
		}
		
		return result;
	};
}

//正規表現でエスケープする文字列をエスケープしてくれる
function preg_quote (str, delimiter) {
    // Quote regular expression characters plus an optional character  
    // 
    // version: 1107.2516
    // discuss at: http://phpjs.org/functions/preg_quote
    // +   original by: booeyOH
    // +   improved by: Ates Goral (http://magnetiq.com)
    // +   improved by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
    // +   bugfixed by: Onno Marsman
    // +   improved by: Brett Zamir (http://brett-zamir.me)
    // *     example 1: preg_quote("$40");
    // *     returns 1: '\$40'
    // *     example 2: preg_quote("*RRRING* Hello?");
    // *     returns 2: '\*RRRING\* Hello\?'
    // *     example 3: preg_quote("\\.+*?[^]$(){}=!<>|:");
    // *     returns 3: '\\\.\+\*\?\[\^\]\$\(\)\{\}\=\!\<\>\|\:'
    return (str + '').replace(new RegExp('[.\\\\+*?\\[\\^\\]$(){}=!<>|:\\' + (delimiter || '') + '-]', 'g'), '\\$&');
}

// strReg ・・・ 許容する文字を正規表現化したもの
// checkStr ・・・ チェック対象文字列
function IsRespectRegCheck(strReg, checkStr)
{
	str = checkStr;

	var re = new RegExp(strReg, 'g');
	var tmp = str.match(re);

	if(tmp != str)
	{
		return false;
	} else {
		return true;
	}
}

//指定されたクラス名のコントロールの有効無効切り替え
/* IE8だとgetElementsByClassNameが動かないので使えない
function disableGroupByClass( classId, ischecked ) {
	var els = document.getElementsByClassName(classId);
	var i;

	for(i=0; i < els.length; i++){
		els[i].disabled = !ischecked;
	}
}
*/

//仕方がないので、IDの配列を引数で渡してもらってIDでdisabledするようにした。
function disableGroupByIds(ids, ischecked)
{
	var el;
	var i;

	for(i=0; i < ids.length; i++)
	{
		el = document.getElementById(ids[i]);
		el.disabled = !ischecked;
	}
}

function escapestr(plainText)
{
		if (!plainText) return "";
		var value = "";
		var dqArray = plainText.split("\"");
		for (var dqIndex = 0; dqIndex < dqArray.length; dqIndex++) {
			if (dqIndex > 0)
				value += "\\\"";
			var yenResult = dqArray[dqIndex].match(/\\+$/);
			if (yenResult) {
				var replaceStr = "";
				for (var yenIndex = 0; yenIndex < yenResult[0].length; yenIndex++)
					replaceStr += "\\\\";
				value += dqArray[dqIndex].replace(/\\+$/g, replaceStr);
			}
			else {
				value += dqArray[dqIndex]
			}
		}

		return value;
} ;

// bat経由で使えない禁則文字定義
var BatExecuteProhibitionChars = "\"<>|&^";

//画面表示用に1文字ずつづスペース区切りで表示する
function ViewStringBatExecuteProhibitionChars() {
	var viewstring = ""
	for (var i = 0; i < BatExecuteProhibitionChars.length; i++) {
		viewstring += (" " + BatExecuteProhibitionChars.charAt(i));
	}
	return viewstring.trim();
}

// bat経由でプログラム実行する際に使えない文字をチェックする
// true:使えない文字が入っている
// false:使えないない文字が入っていない
function IsBatExecuteProhibitionChar(checkText) {
	var reg = new RegExp("[" + BatExecuteProhibitionChars + "]");
	return checkText.match(reg);
}

//禁則チェッククラス
function CProhibitionCharUtils(regstr) {
	//正規表現文字列を保持する
	this.regstr = regstr;

	//画面表示用に1文字ずつづスペース区切りで表示する
	this.ViewString = function () {
		var viewstring = ""
		for (var i = 0; i < this.regstr.length; i++) {
			viewstring += (" " + this.regstr.charAt(i));
		}
		return viewstring.trim();
	}

	// 該当する正規表現文字列が含まれるかチェックする
	// true:使えない文字が入っている
	// false:使えないない文字が入っていない
	this.IsTest = function (checkText) {
		var reg = new RegExp("[" + this.regstr + "]");
		return reg.test(checkText);
	}
}

function CUtils() {
	//末尾にバックスラッシュが存在するかどうか
	this.IsEndBackSlush = function (value) {
		if (value.IsEndWith("\\")) {
			return true;
		}
		return false;
	}

	//末尾がバックスラッシュ場合に警告を表示する
	this.AlertIsEndBackSlush = function (value, errorlabel) {
		if (this.IsEndBackSlush(value)) {
			alert(errorlabel + "の末尾に[\\]は使用できません。");
			return true;
		}
		return false;
	}
}

/***
 * Stringオブジェクトにメソッドを追加
 ***/
if (!String.prototype.IsEndWith) {
	String.prototype.IsEndWith = function (endStr) {
		if (this.substring(this.length - 1) == endStr) {
			return true;
		}
		return false;
	}
}