/**
 * 本ファイルのSHA256ハッシュを
 * Indicatorソースの同値ハッシュと置き換える
 * $filePath = "ファイルパス"; $fileContent = Get-Content -Raw -Encoding Byte -Path $filePath; $hash = [System.Security.Cryptography.SHA256]::Create().ComputeHash($fileContent); $hashBase64 = [Convert]::ToBase64String($hash); $hashBase64
 */
document.addEventListener("DOMContentLoaded", function () {
  // HTTPレスポンスヘッダーから nonce を取得
  fetch(window.location.href).then((response) => {
    // nonce を含むスクリプトタグを動的に作成
    const nonce = response.headers.get("X-CSP-Nonce");

    if (nonce) {
      addNonceToElements('link[rel="stylesheet"]', nonce);
      addNonceToElements("script", nonce);
      addNonceToStyleElements("style", nonce);
    } else {
      console.error("Nonce not found in the response headers.");
    }
  });
});

// 要素に nonce を追加しログを出力する関数
function addNonceToElements(selector, nonce) {
  const elements = document.querySelectorAll(selector);
  elements.forEach((element) => {
    const newElement = element.cloneNode(true);
    newElement.setAttribute("nonce", nonce);
    element.parentNode.replaceChild(newElement, element);
  });
}

function addNonceToStyleElements(selector, nonce) {
  const elements = document.querySelectorAll(selector);
  elements.forEach((element) => {
    const newElement = document.createElement("style");
    newElement.setAttribute("nonce", nonce);
    newElement.textContent = element.textContent;
    element.parentNode.replaceChild(newElement, element);
  });
}

// nonceを追加した要素をログ出力する関数
function logNonceAdded(element, nonce) {
  console.log(
    `Nonce "${nonce}" has been added to ${element.tagName.toLowerCase()}: ${
      element.outerHTML
    }`
  );
}
