import os
import logging
import random
import string
from datetime import datetime, timedelta, timezone
from functools import partial

## 共通使いする定数はここで定義 #############################################################
APP_NAME = "SayAlertFilter"  # アプリ名
JST_HOURS = 9  # JST 時差
KEY_NAME = "crypt_key"  # 暗号キー名
SYSTEM_USER_ID = "SYSTEM"  # SYSTEM USER ID
COOKIE_NAME = "sayalt"  # cookie 名
EXPIRE_DAY: float = 1 / 24  # cookie 有効期限
#############################################################################################


def random_string(length: int = 64) -> str:
    """ランダム文字列取得

    ランダムな文字列を作成する

    Parameters
    ----------
    length:int
        文字列の長さ

    Return
    ------
    str
        生成した文字列
    """
    return "".join(random.choices(string.ascii_letters + string.digits, k=length))


def tzinfo(hours: int = JST_HOURS) -> timezone:
    """タイムゾーン取得

    timezoneの取得

    Parameters
    ----------
    hours : int, optional
        utcからの時差, by default 9(jst)

    Returns
    -------
    timezone
        timezone
    """
    hours = int(os.getenv("tz", hours))
    return timezone(timedelta(hours=hours))


def datetime_now_aware() -> datetime:
    """タイムゾーン付き日時取得

    タイムゾーンを考慮した現在日時を返す

    Returns
    -------
    datetime
        日時
    """
    return datetime.now(tzinfo())


def get_logger(name: str) -> logging.Logger:
    """ロガー取得

    当該システム用のロガーを取得する

    Parameters
    ----------
    name:str
        __name__ の値

    Return
    ------
        対応するロガーインスタンス
    """
    if name == "__main__":
        log_name = APP_NAME
    else:
        log_name = ".".join([APP_NAME, name])

    # 独自レベル定義
    logger = logging.getLogger(log_name)
    logger.detail = partial(logger.log, 5)

    return logger


def to_int(data: any, default: int = 0) -> int:
    """数値返還

    値を int に変換する

    Parameters
    ----------
    data:any
        int に変換する値
    default:int
        変換できない場合の値。省略時は0

    Return
    ------
        int に変換された値
    """
    result: int = 0
    try:
        result = int(data)
    except Exception:
        get_logger(__name__).warning(f"{data} は数値に変換出来ません。{default} が使用されます。")
        result = default

    return result
