
from datetime import datetime, timedelta

from pydantic import constr, Field, confloat

from app import datetime_now_aware, random_string
from app.entity import CommonModel

class AuthInfo(CommonModel):
    """認証データ保存用

    cookie に保存するための認証データ

    """
    user_id: constr(strip_whitespace=True, min_length=1, max_length=32)
    user_name: constr(strip_whitespace=True, min_length=1, max_length=64)
    register_at: datetime = Field(description="登録日時", default_factory=lambda: datetime_now_aware(),)
    expire_time: confloat(gt=0.0, allow_inf_nan=False)
    garbage: str = Field(description="フリーフィールド", default_factory=lambda: random_string(), )

    def is_expire(self) -> bool:
        diff:timedelta = datetime_now_aware() - self.register_at
        return diff.seconds > self.expire_time

    def update(self):
        self.register_at = datetime_now_aware()
        return self
