
from typing import Optional
from pydantic import Field, constr, conint
from datetime import datetime

from app import datetime_now_aware, SYSTEM_USER_ID
from app.enum import MailAuthMethod, MailProtocol, ConnProtect
from app.entity import CommonModel

class MailServer(CommonModel):
    """メール受信設定
    """
    host:constr(strip_whitespace=True, min_length=1, max_length=64)
    port:conint(ge=1, le=65535)
    protocol:conint(ge=min(list(map(int, MailProtocol))), le=max(list(map(int, MailProtocol))))

    use_ssl:conint(ge=min(list(map(int, ConnProtect))), le=max(list(map(int, ConnProtect))))
    auth_method:conint(ge=min(list(map(int, MailAuthMethod))), le=max(list(map(int, MailAuthMethod))))

    user_id:Optional[str] = Field(None, description="ユーザー名")
    password:Optional[str] = Field(None, description="パスワード")
    poll_minute: conint(ge=1, le=1440)

    ref_con: Optional[str] = Field(None, description="フリーエリア")

    update_by:str = Field(SYSTEM_USER_ID, description="更新者ID")
    update_at: datetime = Field(description="更新日時", default_factory=lambda: datetime_now_aware(),)
