
from datetime import datetime

from pydantic import Field, constr, conint

from app import datetime_now_aware, SYSTEM_USER_ID
from app.entity import CommonModel
from app.enum import RoleType
from app.util import convert_password

class Users(CommonModel):
    user_id:constr(strip_whitespace=True, min_length=1, max_length=32)
    user_name:constr(strip_whitespace=True, min_length=1, max_length=64)
    enabled:bool = Field(True, description="アカウント状態")
    secret_key:constr(strip_whitespace=True, min_length=1, max_length=64)
    role:conint(ge=min(list(map(int, RoleType))), le=max(list(map(int, RoleType))))

    update_by:str = Field(SYSTEM_USER_ID, description="更新者ID")
    update_at: datetime = Field(description="更新日時", default_factory=lambda: datetime_now_aware(),)

    def hash_pass(self):
        if self.secret_key:
            self.secret_key = convert_password(self.secret_key)

class ChangePassword(object):
    def __init__(self, user_id:str, old_pass:str, new_pass:str, conf_pass:str, update_by:str):
        self.user_id = user_id
        self.update_by = update_by

        if not user_id:
            raise ValueError("ユーザー ID がありません。")
        elif not old_pass:
            raise ValueError("現在のパスワードがありません。")
        elif not new_pass:
            raise ValueError("新しいパスワードがありません。")
        elif not conf_pass:
            raise ValueError("確認用パスワードがありません。")
        elif new_pass != conf_pass:
            raise ValueError("新しいパスワードが確認用パスワードと一致しません。")
        elif new_pass == old_pass:
            raise ValueError("パスワードが変更されていません。")

        self.old_key = old_pass
        self.secret_key = new_pass

    @property
    def old_key(self):
        if hasattr(self, "_old_key"):
            return self._old_key
        else:
            return ""

    @old_key.setter
    def old_key(self, src_pass:str):
        if src_pass:
            self._old_key = convert_password(src_pass)
        else:
            self._old_key = ""

    @property
    def secret_key(self):
        if hasattr(self, "_new_key"):
            return self._new_key
        else:
            return ""

    @secret_key.setter
    def secret_key(self, src_pass:str):
        if src_pass:
            self._new_key = convert_password(src_pass)
        else:
            self._new_key = ""
