from enum import Enum

"""enum 定義

使用している全ての列挙値を定義

"""


class TimeType(int, Enum):
    """日時タイプ"""

    everyday = 1  # 毎日
    weekday = 2  # 平日
    weekend = 3  # 週末
    workday = 4  # 営業日(将来用)
    holiday = 5  # 休日(将来用)


class FieldType(int, Enum):
    """対象フィールド"""

    field_from = 1  # 差出人
    field_to = 2  # 宛先
    field_cc = 3  # CC
    field_subject = 4  # タイトル
    field_body = 5  # 本文


class TermType(int, Enum):
    """判定条件"""

    included = 1  # ～を含む
    not_included = 2  # ～を含まない
    match_text = 3  # ～と一致する
    not_match_text = 4  # ～と一致しない
    match_regexp = 5  # ～の正規表現とマッチする
    not_match_regexp = 6  # ～の正規表現とマッチしない


class TermJoin(int, Enum):
    """結合条件"""

    join_and = 1  # AND 結合
    join_or = 2  # OR 結合


class MailProtocol(int, Enum):
    """メールプロトコル"""

    pop3 = 1
    smtp = 2


class ConnProtect(int, Enum):
    """接続保護"""

    none = 1  # なし
    starttls = 2  # STARTTLS
    ssl = 3  # SSL


class MailAuthMethod(int, Enum):
    """認証方式"""

    plain = 1  # PLAIN
    apop = 2  # APOP(POP only)
    rpop = 3  # RPOP(POP only)
    none = 4  # NONE
    oauth = 5  # MS-OAuth


class ActionType(int, Enum):
    """フィルター後アクションタイプ"""

    action_mail = 1  # メール
    action_webhook = 2  # WebHook
    action_command = 3  # 外部コマンド


class FlowTerm(int, Enum):
    """流量条件"""

    greater_equ = 1  # n 通以上
    less_equ = 2  # n 通以下


class RoleType(int, Enum):
    """ユーザーロール"""

    admin = 0  # 管理モード
    viewer = 1  # 参照モード


class FileType(int, Enum):
    plain = 1 << 0
    csv = 1 << 1
    json = 1 << 2
    zip = 1 << 3
    cab = 1 << 4
    bin = 1 << 5
    archive = zip | cab


class ResultStatus(str, Enum):
    nothing = "NOTHING"
    error = "ERROR"
    create = "CREATED"
    update = "UPDATED"
    delete = "DELETED"


class ResponseDataType(int, Enum):
    d_null = 0
    d_primitive = 1
    d_list = 2
    d_hash = 3
