
from sqlalchemy import Column, DateTime, String
from sqlalchemy.orm import DeclarativeBase
from sqlalchemy.ext.declarative import declared_attr

from app import datetime_now_aware

class Base(DeclarativeBase):
    def update(self, **kwd:dict[str,any]):
        """アップデートメソッド

        特定フィールドをアップデートする。
        UPDATE_FIELDS リストに対象フィールド名を設定すること。

        Parameters
        ----------
        kwd:dict[str,any]
            アップデート対象のメソッドと値
        """
        for k in self.UPDATE_FIELDS:
            if k in kwd:
                setattr(self, k, kwd[k])

class UpdatesMixin(object):
    @declared_attr
    def update_by(cls):
        """更新者
        """
        return Column(String(32), nullable=False)

    @declared_attr
    def update_at(cls):
        """更新日
        """
        return Column(DateTime(timezone=True), default=datetime_now_aware(), onupdate=datetime_now_aware(), nullable=False)

def modified_at_hook(mapper, connection, target):
    target.update_at = datetime_now_aware()
