
from sqlalchemy import Enum, event
from sqlalchemy import Column, String, Integer

from app.enum import ConnProtect, MailAuthMethod, MailProtocol
from app.models import Base, UpdatesMixin, modified_at_hook

class MailServer(Base, UpdatesMixin):
    """メールサーバー設定
    """
    __tablename__ = "mail_server"
    
    host = Column(String(256), primary_key=True, nullable=False)
    port = Column(Integer, primary_key=True, nullable=False, default=110)
    protocol = Column(Enum(MailProtocol), nullable=False, default=MailProtocol.pop3.name, index=True)
    use_ssl = Column(Enum(ConnProtect), nullable=False, default=ConnProtect.none.name)
    auth_method = Column(Enum(MailAuthMethod), nullable=False, default=MailAuthMethod.plain.name)
    user_id = Column(String(64), nullable=True)
    password = Column(String(256), nullable=True)
    poll_minute = Column(Integer, nullable=False, default=10)

    ref_con = Column(String(1024), nullable=True)

    UPDATE_FIELDS = [
        "use_ssl",
        "auth_method",
        "user_id",
        "password",
        "poll_minute",
        "update_by",
    ]

# event
event.listen(MailServer, "before_insert", modified_at_hook)
event.listen(MailServer, "before_update", modified_at_hook)
