import time
from http import HTTPStatus

from tornado.web import MissingArgumentError

from app import get_logger, COOKIE_NAME
from app.db import get_ses_maker
from app.crud.users import get
from app.router import DefaultBase, SERVER_RUNTIME_ERROR, SaySmallDedaultHandler
from app.util import convert_password
from app.enum import ResponseDataType


class IndexHandler(DefaultBase):
    async def get(self):
        self.redirect("/login")


class LoginHandler(SaySmallDedaultHandler):
    async def get(self):
        # ログイン情報を削除
        self.init_auth_cookie()

        params: dict = DefaultBase.get_common_args()

        self.render("login.html", **params)


class AuthLogin(DefaultBase):
    async def get(self):
        self.redirect("/login")

    async def post(self, *args, **kwarg):
        log = get_logger(__name__)

        try:
            val1 = self.get_argument("user_id", strip=True)
            val2 = self.get_argument("pass_src", strip=True)
            async with get_ses_maker() as db:
                info = await get(db, val1)

            if info is None:
                self.err_response(
                    status_code=HTTPStatus.UNAUTHORIZED,
                    msg_type="E002",
                    msg_body="ユーザー名・パスワードが違います。",
                    detail_type=ResponseDataType.d_null,
                )
                return

            vv = convert_password(val2)
            if info.secret_key != vv:
                log.warning("ユーザー名・パスワードが一致しません。")
                self.err_response(
                    status_code=HTTPStatus.UNAUTHORIZED,
                    msg_type="E002",
                    msg_body="ユーザー名・パスワードが違います。",
                    detail_type=ResponseDataType.d_null,
                )
                return
            else:
                # 認証されたのでクッキーをセット
                self.set_current_user(info.user_id, info.user_name)

            log.info(f"ユーザー ID: {val1} は {self.request.remote_ip} からログインしました。")
        except MissingArgumentError:
            log.exception("認証情報がありません。")
            self.err_response(
                status_code=HTTPStatus.UNPROCESSABLE_ENTITY,
                msg_type="E001",
                msg_body="リクエストパラメータが正しくありません。",
                detail_type=ResponseDataType.d_null,
            )
            return
        except Exception:
            log.exception("認証時にエラーが発生しました。")
            raise SERVER_RUNTIME_ERROR

        self.set_status(HTTPStatus.OK)
        self.write({"status": "OK"})
        self.finish()
