from datetime import timedelta

from tornado.ioloop import PeriodicCallback

from app.entity.mail_server import MailServer
from app.service.pop import PopStore
from app.util import ml_proc

async def start_pops(svr:MailServer):
    """POP3メール受信ポーリング処理登録

    POP3メール受信処理を周期実行する

    Parameters
    ----------
    svr:MailRecv
        メール設定
    """
    key = f"{svr.host}:{svr.port}"
    await stop_pops(svr)  # 動いていたら停止する

    # メール受信設定を開始する
    pop = PopStore(svr)
    ml_proc[key] = PeriodicCallback(pop.mail_check, timedelta(minutes=pop.info.poll_minute))

async def stop_pops(svr:MailServer):
    """POP3メール受信ポーリング処理停止

    POP3メール受信処理の周期実行を終了する

    Parameters
    ----------
    svr:MailRecv
        メール設定
    """
    key = f"{svr.host}:{svr.port}"
    if key in ml_proc:
        ml_proc[key].stop()
        del ml_proc[key]    # 定期実行の設定を破棄する
