from app import get_logger
from app.db import get_ses_maker
from app.crud.mail_filter import get
from app.entity.mail_filter import MailFilter


class MailFilterCache:
    """メールフィルターキャッシュ

    メールフィルターをキャッシュし、都度読み込みをさせない
    """

    def __new__(cls, *args, **kargs):
        """コンストラクタ

        シングルトンクラス用定義
        """
        if not hasattr(cls, "_instance"):
            cls._instance = super(MailFilterCache, cls).__new__(cls)
            cls._filter: list[MailFilter] = []

        return cls._instance

    async def reload(self):
        """フィルターリロード"""
        async with get_ses_maker() as db:
            self._filter = await get(db)

        get_logger(__name__).info("メールフィルターを再読み込みします。")

    def get(self):
        """フィルター取得"""
        for f in self._filter:
            yield f
