import os

from app import get_logger, random_string, SYSTEM_USER_ID
from app.db import DbSession
from app.crud.users import create, get
from app.enum import RoleType
from app.entity.users import Users
from app.util import join_root_path, STORE_ROOT


async def check_sys_users() -> bool:
    """システムユーザー追加

    システムユーザーを追加する。
    その際、デフォルトパスワードをランダムで生成してファイルに出力する。

    Return
    ------
    bool
        正常終了時、True
    """
    log = get_logger(__name__)

    ret_code: bool = False
    DEFAULT_SYS_NAME = "admin"

    # 既存ユーザーの確認
    user = await get(DbSession, DEFAULT_SYS_NAME)
    if user:
        return True

    # パスワードの準備
    passwd = random_string(8)

    # システムユーザーの準備
    user = Users(
        user_id=DEFAULT_SYS_NAME,
        user_name="System Administrator",
        enabled=True,
        secret_key=passwd,
        role=RoleType.admin,
        update_by=SYSTEM_USER_ID,
    )

    result = await create(DbSession, user)
    if result:
        path_name = join_root_path(STORE_ROOT, "INITIAL_SECRET_WORDS")
        try:
            with open(os.open(path_name, os.O_CREAT | os.O_WRONLY | os.O_TRUNC, mode=0o600), "w") as wp:
                wp.write(passwd)
        except Exception:
            log.exception(f"ユーザー {DEFAULT_SYS_NAME} の初期パスワードを {path_name} に保存できませんでした。")
        else:
            log.info(f"ユーザー {DEFAULT_SYS_NAME} の初期パスワードを {path_name} に保存しました。")
            ret_code = True

    return ret_code
